/*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************/

#include <xc.h>

#define FOSC 16000000UL

#include "GenericTypeDefs.h"
#include "i2c.h"
#include "mcp9800.h"

unsigned char delay_tick;

void main(void)
{
    int temperature;
    unsigned char c;
    
    OSCCONbits.IRCF = 15;
        
    // peripheral configuration
    T2CON = 0x00;
    TMR2ON = 1;
    PR2 = 0xFF;
    
    // initialize TMR1
    TMR1H = 255;
    TMR1L = 255 - 33;
    T1CONbits.T1CKPS = 0;

    T1CONbits.TMR1CS = 2;
    T1OSCEN = 1;

    nT1SYNC = 1;
    TMR1ON = 1;
    TMR1IF = 0;
    TMR1IE = 1;

    LCDSE0 = 0;
    LCDSE1 = 0;
    LCDSE2 = 0;
    
    TRISC6 = 0;
    TRISC7 = 0;
    TRISD = 0;
    ANSELD = 0;
    TRISE = 0;
    PORTC = 0;
    PORTE = 0;
    PORTD = 0;
    
    i2c_init();
    
    // enable the interrupts
    PEIE = 1;
    GIE = 1;

    // external HW configuration
    mcp9800_init();
    

	while(1)
	{
        CLRWDT();
        temperature = mcp9800_get_temp();
                
        if(temperature > 1023)
            temperature = 1023;
            
        CCPR3L = temperature >> 2;
        CCP3CON = ((temperature & 0x03) << 4) | 0x0C;

        GIE = 0;
        delay_tick = 10;
        GIE = 1;
        
        do
        {
            GIE = 0;
            c = delay_tick;
            GIE = 1;
        }while(c > 0);

    }   	    
}

void interrupt isr()
{
    if(TMR1IF && TMR1IE)
    {
        TMR1IF = 0;
        TMR1ON = 0;
        TMR1H = 0xff;
        TMR1L = 255 - 33;
        TMR1ON = 1;
        RD1 = !RD1;
        if(delay_tick)
            delay_tick --;
    }    
    i2c_handler();
}
