
/********************************************************************
 *                                                                     
 *                     Software License Agreement                      
 *                                                                     
 * Microchip Technology Inc. (Microchip) licenses this software to
 * you solely for use with Microchip microcontrollers and Microchip
 * serial EEPROM products.  The software is owned by Microchip and/or
 * its licensors, and is protected under applicable copyright laws.
 * All rights reserved.
 *
 * SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
 * DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
 * LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
 *                                                                     
 ********************************************************************
 *
 *   Filename:           UNIO PIC16.h
 *   Processor:          PIC16F616
 *   Compiler:           HI-TECH C PRO for PIC10/12/16 V9.60PL4
 *   Date:               March 29, 2010
 *   File Version:       1.0
 *
 *   Author:             Chris Parris
 *   Company:            Microchip Technology Inc.
 *                                                                    
 ********************************************************************
 *
 *   Modified
 *   Version   Author    Date        Comments
 *   1.1      w.brown    2012.02.07  Define _XTAL_FREQ if not already defined
 *******************************************************************/

#ifndef	__UNIO_PIC16_H
#define	__UNIO_PIC16_H

/** C O N S T A N T   D E F I N I T I O N S ************************/
#ifndef _XTAL_FREQ
#define _XTAL_FREQ  32000000L           // 32 MHz oscillator
#endif
#define TIMER1_PRESCALE  4L             // Prescale needed to emulate 8 MHz _XTAL_FREQ
#define STARTHDR    0x55                // Define STARTHDR as 0x55
#define SAK         1                   // Define SAK as 1
#define NOSAK       0                   // Define NOSAK as 0
#define MAK         1                   // Define MAK as 1
#define NOMAK       0                   // Define NOMAK as 0
#define DEVICE_ADDR 0xA0                // Define DEVICE_ADDR as 0xA0
#define PAGESIZE    16                  // 16-byte page size for 11XXXXX

/** B U S   F R E Q U E N C Y   C A L C U L A T I O N **************/
#define BUSFREQ     10000L               // Set desired bus frequency to 10 kHz
#define HALF_PERIOD (((_XTAL_FREQ/(TIMER1_PRESCALE*8L))/BUSFREQ)-1)// Timer period calculation

/** C O M M A N D   D E F I N I T I O N S **************************/
#define WREN        0b10010110          // WREN command
#define WRDI        0b10010001          // WRDI command
#define WRITE       0b01101100          // WRITE command
#define READ        0b00000011          // READ command
#define WRSR        0b01101110          // WRSR command
#define RDSR        0b00000101          // RDSR command
#define ERAL        0b01101101          // ERAL command
#define SETAL       0b01100111          // SETAL command

/** I / O   P I N   D E F I N I T I O N S **************************/
#define SCIO        RG3                 // Serial clock, input/output pin, PORTG pin 3
#define SCIOTRIS    TRISG3              // SCIO direction bit

/** M A C R O S ****************************************************/
// WAIT_CCP1IF is equivalent to "while(CCP1IF == 0) {};" - the macro
//   was defined because the version of that line when compiled in
//   Lite mode is extremely inefficient.
#define WAIT_CCP1IF while(!CCP1IF);
//				    asm("bcf _STATUS,5"); \
//                    asm("btfss _PIR1,5"); \
//                    asm("goto $-1");


/** T Y P E D E F S ************************************************/
typedef union _FLAGS
{
    unsigned char _byte;
    struct
    {
        unsigned sendMAK:1;
        unsigned checkSAK:1;
    };
} FLAGS;

typedef union _STATUSREG
{
    unsigned char _byte;
    struct
    {
        unsigned WIP:1;
        unsigned WEL:1;
        unsigned BP0:1;
        unsigned BP1:1;
    };
} STATUSREG;

typedef union _BYTE
{
    unsigned char _byte;
    struct
    {
        unsigned b0:1;
        unsigned b1:1;
        unsigned b2:1;
        unsigned b3:1;
        unsigned b4:1;
        unsigned b5:1;
        unsigned b6:1;
        unsigned b7:1;
    };
} BYTE;

typedef union _WORD
{
    unsigned int _word;
    struct
    {
        unsigned char byte0;
        unsigned char byte1;
    };
    struct
    {
        BYTE Byte0;
        BYTE Byte1;
    };
    struct
    {
        BYTE LowB;
        BYTE HighB;
    };
    struct
    {
        unsigned char v[2];
    };
} WORD;
#define LSB(a)      ((a).v[0])
#define MSB(a)      ((a).v[1])

/** P R O T O T Y P E S ********************************************/
void ByteWrite(WORD);                           // Byte Write function
void ByteRead(WORD);                            // Byte Read function
void PageWrite(WORD,unsigned char);             // Page Write function
void SeqRead(WORD,unsigned char);               // Sequential Read function
STATUSREG ReadStatusReg(void);                  // Read Status Register function
void WriteStatusReg(STATUSREG);                 // Write Status Register function
void WriteEnable(void);                         // Write Enable function
void WriteDisable(void);                        // Write Disable function
void WIP_Poll(void);                            // WIP polling function
void StandbyPulse(void);                        // Generate Standby Pulse
void UNIO_init(void);                           // Init. function
void UNIO_restore(void);                        // Restore registers altered by Init function
void SetAll(void);                              // Set all locations to 0xFF

extern unsigned char pageBuffer[PAGESIZE];      // 16-byte page buffer

#endif
