/*///////////////////////////////////////////////////////////////////////////
//
// File: MonitorTests.c
//
// C based tests callable from the monitor program with the T command.
//
/////////////////////////////////////////////////////////////////////////////
*                
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment
*
* w.r.brown            2011.08.10  V 8    - Added stepper support
* w.r.brown            2011.05.11  V 7    - Added UNIO support
* w.r.brown            2011.05.02  V 6    - Tests 3 and 4 obsolete. Removed incremental speed changes.
* w.r.brown            2010.10.01  V 5    - Tests 5 and 6 obsolete. Advance and balance changed to degrees.
* w.r.brown            2010.08.18  V 5    - Added test 19
* w.r.brown            2010.07.23  V 4    - Added tests 15-18
* w.r.brown            2010.07.14  V 3    - Started keeping track of changes.
*******************************************************************************************************/

#define TEST_VERSION ".8 "

#include <xc.h>
#include "Main.h"
#include "GenericTypeDefs.h"
#include "EEData.h"
#include "Monitor.h"
#include "UartInterface.h"
#include "Init.h"
#include "UNIO PIC16.h"
#include "UNIO_support.h"
#include "Stepper.h"

void SendAddresses(void);
void SendWord(unsigned int db, char n);
void WriteChar(char x);

#include "MonitorTests.h"

const char rsTestVersion[] = TEST_VERSION;

bit StepReady;

extern volatile doublebyte InputData;
//extern doublebyte ADC;
extern bit SpeedReady;
extern bit OffsetReady;
extern bit GetCommTime;
extern bit BalanceReady;
extern bit BrakeRequest;
extern bit CCPValReady;

volatile doublebyte speed_increment;
extern volatile unsigned int CommTime;

extern unsigned char blanking_count;
extern bit OpenLoopFlag;
extern bit SingleStep;
extern bit ReverseDirection;
extern volatile char startup_dutycycle;
extern volatile unsigned int startup_comm_time;
extern volatile unsigned int max_timer1_preset;
extern bit             TXON;
extern bit             stop_flag;
extern bit             run_flag;
extern doublebyte      TMR1_comm_time;
extern unsigned char   Poles;
extern unsigned char   SlowStartDwell;
extern unsigned char   OpenLoopDwell;
extern unsigned char   error_scale;
extern unsigned char   speed_request_off;
extern unsigned char   speed_request_on;
extern char            CommOffsetRequest;
extern char            TimingBalanceRequest;
extern unsigned char   BrakeLevel;
extern char            cTemp;
extern unsigned char   add_on_revision;
extern mode_motor      motor_type;
extern mode_step       step_mode;
extern unsigned char   stepSize;
extern unsigned int    stepInc;
extern unsigned int    stepCount;
extern unsigned char   DutyScale;
extern unsigned char   MinDuty;
extern unsigned char   MicroStepSize;
extern unsigned long   SpeedScalingConst;
extern unsigned long   SpeedConstTimer;
extern unsigned char   CCP_Val[];

//const char rsTest0[]         = " Test 0\r";
const char sInputData[]      = "\rDATA:";
const char sSpeedIncr[]      = "\rSINC:";
const char sStartDutyCycle[] = "\rSDCY:";
const char sStartCommTime[]  = "\rSTIM:";
const char sCCPR1L[]         = "\rCCPR:";
const char sOffset[]         = "\rOFST:";
const char sBlnkCnt[]        = "\rBLNK:";
const char sStall[]          = "\rSTAL:";
const char sPoles[]          = "\rPOLS:";
const char sSSDwell[]        = "\rSLOW:";
const char sOLDwell[]        = "\rOPNL:";
const char sSpeedReqOn[]     = "\rSRON:";
const char sSpeedReqOff[]    = "\rSROF:";
const char sErrorScale[]     = "\rERRS:";
const char sBalance[]        = "\rBLNC:";
const char sBrake[]          = "\rBRKE:";
const char sType[]           = "\rTYPE:";
const char sMinDuty[]        = "\rMDTY:";
const char sDutyScale[]      = "\rDTYS:";
const char sSpeedConstTimer[]= "\rSCTM:";
const char sSpeedScalingConst[]= "\rSSCT:";
const char sMicroStepSize[]  = "\rSTEP:";
const char sStepMode[]       = "\rMODE:";
const char sCCPTable[]       = "\rTABL:";
const char sNewLine[]        = "\r>";
char sOutput[50];

/////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////// SUBROUTINES //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------------
void Test(unsigned char t)
{
	static unsigned int addr;
	// general purpose user defined tests
	
	switch (t)
	{
		case 0:
	        step_mode = WAVE;
	        stepSize = 0;
	        EEDATL = 0;  // this signals the speed manager to resend the Timer1 period
	        break;
		case 1:
		   // set flag to transfer InputData to CommOffset
		   OffsetReady = 1;
		   break;
		case 2:
		   // set flag to transfer InputData to ADC simulator
		   SpeedReady = 1;
		   break;
		case 3:
	        step_mode = FULL_STEP;
	        stepSize = 0;
	        EEDATL = 0;  // this signals the speed manager to resend the Timer1 period
	        break;
	    case 4:
	        step_mode = HALF_STEP;
	        stepSize = 1;
	        EEDATL = 0;  // this signals the speed manager to resend the Timer1 period
	        break;
	    case 5:
	        step_mode = MICRO_STEP;
	        stepSize = MicroStepSize;
	        stepInc = LOOK_UP_TABLE_SIZE>>stepSize;
	        stepCount = 0;
	        EEDATL = 0;  // this signals the speed manager to resend the Timer1 period
	        break;
	    case 6:
		    // preload EEDATL with desired step_size (2^step_size = number of steps in one commutation phase)
		    // StepReady signals to make the change.
	        StepReady=1;
	        break;
	    case 7:
		   SendAddresses();
		   break;
		case 8:
		   // enter open loop mode
		   OpenLoopFlag = 1;
		   run_flag = 1;
		   break;
		case 9:
		   // enter closed loop mode
		   OpenLoopFlag = 0;
		   stop_flag = 1;
		   break;
		case 10:
		   // single step open loop
		   SingleStep = 1;
		   break;
		case 11:
		   // force stop condition
		   InputData.word = 0;
		   SpeedReady = 1;
		   break;
		case 12:
		   // reverse
		   ReverseDirection = 1;
		   break;
		case 13:
		   // forward
		   ReverseDirection = 0;
		   break;
		case 14:
		   // capture current TMR1_comm_time
		   if(!OpenLoopFlag)
		   {
	   		   if(run_flag)
	   		   {
			         GetCommTime = 1;
			         while(GetCommTime);
	 		         SendWord(CommTime,4);
	   		   }
	   		   else
	   		      SendWord(0,4);
			}
			else
				SendWord((unsigned int)TMR1_comm_time.word,4);
 			WriteString((char*)sNewLine);

		   break;
		case 15:
		   // save all motor parameters in EEPROM
	      NV_Variables_Save();
	      break;
		case 16:
		   // restore all motor parameters from EEPROM
	      NV_Variables_Read();
	      break;
		case 17:
		   // restore all motor defaults to RAM and EEPROM
	      set_defaults();
	      break;
		case 18:
		   // set flag to transfer InputData to timing_balance
		   BalanceReady = 1;
		   break;
		case 19:
		   // set drive voltage via GetCCP1Val()
		   // use this in OpenLoop mode instead of setting CCPR1L directly
		   switch (motor_type)
		   {
    		   default:
    		   case BLDC:
    		   case BDC:
		            GetCCPVal(InputData.bytes.low);
		            break;
		       case UNIPOLAR:
		       case BIPOLAR:
		            // EEDATL was used because it is easier to access with the Communicator tool
                	EEDATL = InputData.bytes.low;
                	// set flag to signal ISR to reload the duty cycle registers
                	CCPValReady = 1;
                	break;
           }
		   break;
      case 20:
         // braking on
         BrakeRequest = 1;
         break;
      case 21:
         // braking off
         BrakeRequest = 0;
         break;	
	  case 22:
	  	 // Sleep with wake on break
	  	 WUE = 1;
	  	 #asm
	  	 sleep	  	 
	  	 #endasm
	  	 // wait for break to end then clear RCIF
	  	 while(WUE == 1);
	  	 cTemp = RCREG;
	  	 break;
	  case 23:
	     // Initialize UNIO peripherals
	     UNIO_init();
	  	 break;
	  case 24:
	     // restore peripheral settings
	     UNIO_restore();
	     break;
	  case 25:
	     // write data in EEDATL to UNIO address at EEADRH:L
	     addr = EEADRL;
	     addr += (EEADRH & 0x7F)<<8;
	     WriteUNIOByte(addr, EEDATL);
	     break;         	   
	  case 26:
	     // read data in UNIO address EEADRH:L into EEDATL
	     addr = EEADRL;
	     addr += (EEADRH & 0x7F)<<8;
	     EEDATL = ReadUNIOByte(addr);
	     break;         	   
	  case 27:
	     // read UNIO status register into EEDATL
	     EEDATL = ReadStatusReg()._byte;
	     break;
	  case 28:
	     // Write EEDATL into UNIO status register
	     WriteUNIOStatus(EEDATL);
	     break;
	  case 29:
	     // Write 0xFF to all locations
	     SetAll();
	     break;
	  case 30:
	     // read with auto increment
	     // first initialize addr with Test26 or 25
	     EEDATL = ReadUNIOByte(++addr);
	     EEADRH = addr>>8;
	     EEADRL = addr&0xFF;
         break;	     
	  default:
		 break;
	}
}
//------------------------------------------------------------------------------------
void WriteString(char *p)
{
	// suspend program operation while the string at p is written
	// to the USART.
	
	while(*p != 0x00)
	{
		if(TxReady()) PutTx(*p++);
		TxService();
	}
}
//------------------------------------------------------------------------------------
char* uitoa(unsigned int i)
{
	// convert unsigned integer to ascii string
	
	char *p;
	
	p = sOutput + sizeof(sOutput)-1;
	
	*p = 0x00;	// initialize with null termination
	
	do
	{
		p--;
		*p = (i % 10) + '0';
		i /= 10;
	}while (i>0);
	
	return p;
}

void SendAddresses(void)
{
   // send variable addresses
	
	WriteString((char*)sInputData);
	SendWord((unsigned int)&InputData,3);
	WriteString((char*)sSpeedIncr);
	SendWord((unsigned int)&speed_increment,3);
	WriteString((char*)sStartDutyCycle);
	SendWord((unsigned int)&startup_dutycycle,3);
	WriteString((char*)sStartCommTime);
	SendWord((unsigned int)&startup_comm_time,3);
	WriteString((char*)sCCPR1L);
	SendWord((unsigned int)&CCPR1L,3);
	WriteString((char*)sOffset);
	SendWord((unsigned int)&CommOffsetRequest,3);
	WriteString((char*)sBlnkCnt);
	SendWord((unsigned int)&blanking_count,3);
	WriteString((char*)sStall);
	SendWord((unsigned int)&max_timer1_preset,3);
	WriteString((char*)sPoles);
	SendWord((unsigned int)&Poles,3);
	WriteString((char*)sSSDwell);
	SendWord((unsigned int)&SlowStartDwell,3);
	WriteString((char*)sOLDwell);
	SendWord((unsigned int)&OpenLoopDwell,3);
	WriteString((char*)sSpeedReqOn);
	SendWord((unsigned int)&speed_request_on,3);
	WriteString((char*)sSpeedReqOff);
	SendWord((unsigned int)&speed_request_off,3);
	WriteString((char*)sErrorScale);
	SendWord((unsigned int)&error_scale,3);
	WriteString((char*)sBalance);
	SendWord((unsigned int)&TimingBalanceRequest,3);
	WriteString((char*)sBrake);
	SendWord((unsigned int)&BrakeLevel,3);
	WriteString((char*)sType);
	SendWord((unsigned int)&motor_type,3);
	WriteString((char*)sMinDuty);
	SendWord((unsigned int)&MinDuty,3);
	WriteString((char*)sDutyScale);
	SendWord((unsigned int)&DutyScale,3);
	WriteString((char*)sSpeedConstTimer);
	SendWord((unsigned int)&SpeedConstTimer,3);
	WriteString((char*)sSpeedScalingConst);
	SendWord((unsigned int)&SpeedScalingConst,3);
	WriteString((char*)sMicroStepSize);
	SendWord((unsigned int)&MicroStepSize,3);
	WriteString((char*)sStepMode);
	SendWord((unsigned int)&step_mode,3);
	WriteString((char*)sCCPTable);
	SendWord((unsigned int)CCP_Val,4);
   // terminate message
	WriteString((char*)sNewLine);
	// empty the transmit buffer
   while (TXON) TxService();
}

void SendWord(unsigned int db, char n)
{
   // send the n nybble ascii representation of db to serial output
   
	union SPLIT_BYTE sb;
	struct NYBBLE temp;
   
	sb.val = db>>8;
	if(n>3)
	{
      temp.val = sb.nybbles.ms;
      WriteChar(HexToAscii(temp));
    }
	temp.val = sb.nybbles.ls;
    WriteChar(HexToAscii(temp));
    sb.val = db & 0xFF;
    temp.val = sb.nybbles.ms;
    WriteChar(HexToAscii(temp));
	temp.val = sb.nybbles.ls;
	WriteChar(HexToAscii(temp));
}

void WriteChar(char x)
{
   // keep transmitting until some buffer space opens up
   while (!TxReady())
   {
      CLRWDT();
      TxService();
   } 
 	PutTx(x);
}         
