/////////////////////////////////////////////////////////////////////////////
//
// File: UartInterface.h
//
// USART wrappers for C based monitor program.
//
/////////////////////////////////////////////////////////////////////////////

#define nvSPBRG                          0xFF

enum IMMEDIATE {BREAK, ESCAPE, END_OF_TEST, NORMAL};

void Autobaud(void);
void WaitForRise(void);
void SenseBreak(void);
void PutcUsart(char c);
void USARTInit(void);
void PutTx(unsigned char);
void TxService(void);
void ResetRxPointers(void);
void ResetRxTx(void);
enum IMMEDIATE RxService(void);
unsigned char GetRx(void);
char TxReady(void);
void WriteTimer0(unsigned int x);

#ifndef T0IF
#define T0IF TMR0IF
#endif

// bit and bitfield definitions for RC1STA
// 9th bit of received data (can be parity bit)
volatile bit RX9D                @ ((unsigned)&RC1STA*8)+0;
// Overrun Error bit
volatile bit OERR                @ ((unsigned)&RC1STA*8)+1;
// Framing Error bit
volatile bit FERR                @ ((unsigned)&RC1STA*8)+2;
// Address Detect Enable bit
volatile bit ADDEN               @ ((unsigned)&RC1STA*8)+3;
// Continuous Receive Enable bit
volatile bit CREN                @ ((unsigned)&RC1STA*8)+4;
// Single Receive Enable bit
volatile bit SREN                @ ((unsigned)&RC1STA*8)+5;
// 9-bit Receive Enable bit
volatile bit RX9                 @ ((unsigned)&RC1STA*8)+6;
// Serial Port Enable bit
volatile bit SPEN                @ ((unsigned)&RC1STA*8)+7;

// bit and bitfield definitions for TX1STA
// 9th bit of transmit data; can be used as parity bit
volatile bit TX9D                @ ((unsigned)&TX1STA*8)+0;
// Transmit Operation Idle Status bit
volatile bit TRMT                @ ((unsigned)&TX1STA*8)+1;
// High Baud Rate Select bit
volatile bit BRGH                @ ((unsigned)&TX1STA*8)+2;
// Send BREAK character bit
volatile bit SENDB               @ ((unsigned)&TX1STA*8)+3;
// USART Mode Select bit
volatile bit SYNC                @ ((unsigned)&TX1STA*8)+4;
// Transmit Enable bit
volatile bit TXEN                @ ((unsigned)&TX1STA*8)+5;
// 9-bit Transmit Enable bit
volatile bit TX9                 @ ((unsigned)&TX1STA*8)+6;
// Clock Source Select bit
volatile bit CSRC                @ ((unsigned)&TX1STA*8)+7;

// bit and bitfield definitions for BAUD1CON
// Auto-Baud Detect Enable bit
volatile bit ABDEN               @ ((unsigned)&BAUD1CON*8)+0;
// Wake-Up Enable bit
volatile bit WUE                 @ ((unsigned)&BAUD1CON*8)+1;
// 16-bit Baud Rate Register enable bit
volatile bit BRG16               @ ((unsigned)&BAUD1CON*8)+3;
// Clock/Transmit Polarity Select bit
volatile bit SCKP                @ ((unsigned)&BAUD1CON*8)+4;
// Receive Operation Idle Status bit
volatile bit RCIDL               @ ((unsigned)&BAUD1CON*8)+6;
// BRG Rollover Status bit
volatile bit ABDOVF              @ ((unsigned)&BAUD1CON*8)+7;


