/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : BDC.c                                      *
*     Date                 : 2011/05/20                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    BDC add-on board routines for use with F1 LV Evaluation board.                                  //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
********************************************************************************************************
* Change History:
* Author               Date        Comment
*
* w.r.brown            2011.05.20  V 0.1   - original
* w.r.brown            2011.06.07  V 0.2   - added reverse control to BDCInit
* w.r.brown            2011.06.08  V 0.3   - added ReadBDCParameters
* w.r.brown            2011.07.21  V 0.4   - added run_mode init for startup state machine
********************************************************************************************************
*/

#include <xc.h>
#include "Main.h"
#include "BDC.h"
#include "EEData.h"
#include "UNIO_support.h"

extern bit startup_complete_flag;
extern bit ReverseDirection;
extern unsigned char speed_request_off;
extern unsigned char speed_request_on;
extern mode_run run_mode;

/************************************************************************
*                                                                       *
*      Function:       BDCInit                                          *
*                                                                       *
*      Description:    Setup ECCP2 to drive motor in forward direction  *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void BDCInit(void) 
{
	startup_complete_flag = 1;
	CCPR2L = 0x00;
	CCP2CON = 0b01001100;
	P2M1 = ReverseDirection;
	run_mode = RUN;
}

/************************************************************************
*                                                                       *
*      Function:       void ReadBDCParameters(void)                     *
*                                                                       *
*      Description:    Reads user definable BDC motor parameters from   *
*                      UNIO memory                                      *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void ReadBDCParameters(void) 
{
	// adjust Timer1 and CCP1 for UNIO control
	InitUNIO();
	
	speed_request_off = ReadUNIOByte(nvSpeedRequestOff);
	speed_request_on = ReadUNIOByte(nvSpeedRequestOn);
	
	// restore Timer1 and CCP1
	RestoreUNIO();
	
}