/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : Stall_CCP.c                                *
*     Date                 : 2010/07/13                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    BLDC stall detection routine when CCP4 is used as commutation timer.                            //
//                                                                                                    //
//    The commutation lock algorithm will naturally ramp up to a speed above the motor maximum        //
//    when the motor is stalled. This can be detected by comparing the commutation period with        //
//    a predetermined minimum. When the commutation period is less than that minimum then the         //
//    motor is considered stalled and the stop flag is set.                                           //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*/

#include <xc.h>
#include "Main.h"

extern bit OpenLoopFlag;
extern bit startup_complete_flag;
extern unsigned char TMR0_stallcheck_timer;
extern unsigned char TMR0_stall_timer;
extern doublebyte TMR1_comm_time;
volatile unsigned int min_timer1_preset;
extern bit stop_flag;

/************************************************************************
*                                                                       *
*      Function:       StallControl                                     *
*                                                                       *
*      Description:     check motor stall and re-start                  *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*     StallControl is called at the 10 ms time tick set by              *
*     the TimebaseManager and TMR0_run_state_flag                       *
*                                                                       *
*************************************************************************/

void StallControl(void) 
{   
   if(OpenLoopFlag) return;             // ignore stall conditions in open loop

   // TMR0_stall_timer is preset every stable detect event
   // if no stable events are detected then the timer will 
   // time-out forcing the control into a stop condition.
   if(--TMR0_stall_timer == 0)
   {
      stop_flag=1;
   }   
   // check other stall indicators less often
   if(--TMR0_stallcheck_timer==0)
   {
      TMR0_stallcheck_timer = TIMEBASE_STALLCHECK_COUNT;
      if((unsigned int)TMR1_comm_time.word < min_timer1_preset)
      {
         stop_flag = 1;
      }   
   }
}
