/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : F1947_Init_CCP.c                           *
*     Date                 : 2010/07/09                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    BLDC initialization routines for PIC16F1947.                                                    //
//    These are the standard routines used in all software versions up to this point.                 //
//    This is the first time they have been split off into a seperate file.                           //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
********************************************************************************************************
* Change History:
* Author               Date        Comment
*
* w.r.brown            2011.08.08  V 0.6   - added stepper support 
* w.r.brown            2011.07.21  V 0.5   - Restructured to replace semiphore driven startup with state machine 
* w.r.brown            2011.05.23  V 0.4   - Changed InitDriver() to initialize based on motor type. 
* w.r.brown            2010.08.16  V 0.3   - Added call to SlowStartInit()in InitSystem(). 
* w.r.brown            2010.08.16  V 0.2   - Cleared PWM steering registers during Init_IO().
* w.r.brown            2010.08.12  V 0.1   - Derived from F1937_Init_CCP.c
********************************************************************************************************
*/

#include <xc.h>
#include "Main.h"
#include "GenericTypeDefs.h"
#include "Startup.h"
#include "Init.h"
#include "BrakeControl.h"
#include "BDC.h"
#include "Stepper.h"

extern unsigned char comm_state;

extern doublebyte TMR1_comm_time;

extern char slow_start_events;
extern unsigned int TMR0_run_state_timer;
extern unsigned char TMR0_stall_timer;
extern unsigned char TMR0_duty_timer;
extern unsigned char TMR0_supply_timer;
extern unsigned char TMR0_stallcheck_timer;
extern unsigned char timebase_10ms;
extern unsigned char timebase_duty_count;

extern bit TMR0_run_state_flag;
extern bit TMR0_duty_flag;
extern bit TMR0_brake_flag;
extern bit startup_complete_flag;
extern bit stop_flag;
extern bit run_flag;
extern bit measure_bemf_flag;
extern bit startup_in_progress;
extern bit ReverseDirection;
extern bit SpeedReady;
extern bit OffsetReady;
extern bit GetCommTime;
extern bit BalanceReady;

extern int zc_error;
extern int temp;
extern char cTemp;
extern doublebyte expected_zc;
extern doublebyte zc;
extern doublebyte comm_after_zc;
extern unsigned char ramped_speed;
extern volatile unsigned int min_timer1_preset;
extern volatile unsigned int max_timer1_preset;
extern int Offset15;

extern volatile char startup_dutycycle;
extern volatile unsigned int startup_comm_time;

extern bit OpenLoopFlag;
extern unsigned char SlowStartDwell;
extern unsigned char SlowStartSteps;
extern unsigned char OpenLoopDwell;
extern const int CCP_Values[256];
extern mode_motor  motor_type;
extern mode_run run_mode;

/************************************************************************
*                                                                       *
*      Function:       InitSystem                                       *
*                                                                       *
*      Description:    initialize system registers and variables        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void InitSystem(void) {

   // disable all interrupts
   
   INTCON = 0;
   PIE1 = 0;
   PIE2 = 0;
   
   // Setup internal oscillator frequency
   OSCCON = OSCCON_INIT;
   
   //OPTION=OPTION_INIT;   

   Init_IO();
     
   // initialize ADC

   ADCON0 = ADCON0_INIT;
   ADCON1 = ADCON1_INIT;
   
   //disable PWM
   
   CCP1CON = 0;
   CCP2CON = 0;
   CCP3CON = 0;
   CCPR1L = 0;
   CCPR2L = 0;
   CCPR3L = 0;

   // TIMER0 and related startup variables
   TMR0 = 0;
   T0IF = 0;
   OPTION_REG = OPTION_REG_INIT;
   TMR0_run_state_timer = TIMEBASE_WARMUP_COUNT;
   TMR0_supply_timer = TIMEBASE_SUPPLY_COUNT;
   TMR0_duty_timer = timebase_duty_count;
   TMR0_stallcheck_timer = TIMEBASE_STALLCHECK_COUNT;
   TMR0_stall_timer = TIMEBASE_STALL_COUNT;
   timebase_10ms = TIMEBASE_LOAD_10ms;
   slow_start_events = SlowStartSteps;
   ramped_speed = 0;
   
   // Timebase manager flags
   TMR0_run_state_flag = 0;
   TMR0_duty_flag = 0;
   TMR0_brake_flag = 0;
   
   // TIMER1 
   T1CON = T1CON_INIT;

   // COMPARATORS
   CMxCON0 = CMxCON0_INIT;
   //CMyCON0 = CMyCON0_INIT;

   // DRIVER init
   PR2 = PWM_PERIOD;
   T2CON = T2CON_INIT;
   
   // clear all flags
   measure_bemf_flag = 0;
   zc_error = 0;
   SpeedReady = 0;
   OffsetReady = 0;
   GetCommTime = 0;
   BalanceReady = 0;
   startup_complete_flag = 0;
   stop_flag = 0;
   run_flag = 0;
   
   // detect stall by sensing too fast commutation
   
   min_timer1_preset = (0xFFFF - max_timer1_preset)+1;
   
   SlowStartInit();
   //BrakeInit();
   run_mode = WARMUP;
}

/************************************************************************
*                                                                       *
*      Function:       WarmUpControl                                    *
*                                                                       *
*      Description:    Minimum wait period between start and run        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  This is the dwell time between system intialization and application  *
*  of power to the motor by InitDriver().                               *
*                                                                       *
*  Warmup period is controlled by the TimeBaseManager(). Nothing in the *
*  system starts until the warmup period of TIMEBASE_WARMUP_ms has      *
*  elapsed.                                                             *
*                                                                       *
*************************************************************************/

void WarmUpControl(void)
{
   if(TMR0_run_state_timer) 
   {
      TMR0_run_state_timer--;
   }
   else
   {

      // the SpeedManager() determines when the run_flag goes true
      // wait until commanded by the speed control before completing warmup
      // and progressing to the next stages of initializing and startup
      if(run_flag)
      {
         InitDriver();         
      }
   }
}

/************************************************************************
*                                                                       *
*      Function:       InitDriver                                       *
*                                                                       *
*      Description:    Initializes the peripherals appropriate for      *
*                      the detected add-on board                        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  Drivers are initialized after the warmup period and when restarting  *
*  after a stop or stall event.                                         *
*                                                                       *
*************************************************************************/

void InitDriver(void) 
{
	switch (motor_type)
	{
    	case BLDC:
			BLDCInit();        
           	break;
    	case BDC:
    		BDCInit();           
           	break;
       	case UNIPOLAR:           
       	case BIPOLAR:
       	    StepperInit();
       	    break;           
		//case NONE:
       	//case TEST:
       	default:
           	break;
	}
}

/************************************************************************
*                                                                       *
*      Function:       RunControl                                       *
*                                                                       *
*      Description:    Direct any startup and run requirements          *
*                      specific to the detected motor.                  *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:  Startup commences after intialization                     *
*                                                                       *
*                                                                       *
*************************************************************************/

void RunControl(void) 
{
    // TMR0_run_state_flag is a 10 ms time tick set by the TimebaseManager
    if(!TMR0_run_state_flag) return;
       
    TMR0_run_state_flag = 0;
    
    switch(run_mode)
    {
	    case WARMUP:
	        WarmUpControl();
	        break;
	    case SLOWSTART:
            ControlSlowStart();
	        break;
	    case ACCELERATE:
            ControlStartup();
	        break;
	    case STALLDETECT:
            StallControl();
	        break;
	    default:
	    case RUN:
	        break;
    }
}

/************************************************************************
*                                                                       *
*      Function:       BLDCInit(void)                                   *
*                                                                       *
*      Description:    Initialize PWM and driver registers & variables  *
*                      for BLDC operation                               *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  Drivers are initialized after the warmup period and when restarting  *
*  after a stop or stall event.                                         *
*                                                                       *
*************************************************************************/

void BLDCInit(void) 
{
   // suppress minimum duty cycle when in open loop mode
//   if(!OpenLoopFlag)
//   {
//      // startup duty cycle is set by SupplyManager()
//      ramped_speed = FindTableIndex(startup_dutycycle);
//      GetCCPVal(ramped_speed);
//   }
//   CCPR1L = 0x00;
   CCP1CON = CCP1CON_INIT;      //    PWM1 on
   CCP2CON = CCP1CON_INIT;      //    PWM2 on
   CCP3CON = CCP1CON_INIT;      //    PWM3 on
#ifdef ECCPAS_INIT
   ECCPAS = ECCPAS_INIT;        // autoshutdown mode
   PWM1CON = PWM1CON_INIT;      // restart mode
#endif

   TMR1_comm_time.word = -startup_comm_time;
   
   // expected_zc is positive elapsed time at zero cross event
   // strip extended sign bit to keep it positive
   expected_zc.word = (TMR1_comm_time.word>>1) & 0x7FFF;
   zc.word = expected_zc.word;
   comm_after_zc.word = expected_zc.word;
   Offset15 = expected_zc.word>>1;
           
//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Initial Commutation State /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

#ifdef HIGH_SIDE_MODULATION
// startup state plus slow steps must be even for high side modulation
   if((SlowStartSteps & 0x01) && !ReverseDirection)
      comm_state = 1;
   else
      comm_state = 2;
#else
// startup state plus slow steps must be odd for low side modulation
   if((SlowStartSteps & 0x01) && !ReverseDirection)
      comm_state = 2;
   else
      comm_state = 1;
#endif

    run_mode = SLOWSTART;
    TMR0_run_state_timer = SlowStartDwell;         

}

/************************************************************************
*                                                                       *
*      Function:       Init I/O                                         *
*                                                                       *
*      Description:     initialize I/O Ports                            *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  Analog, Tristate, and initial PORT driver states                     *
*                                                                       *
*************************************************************************/

void Init_IO(void)
{
   // set all ports tristate
   
   TRISA = 0xFF;
   TRISB = 0xFF;
   TRISC = 0xFF;
   
   // initialize port analog 

   ANSELA=ANSELA_INIT;
   ANSELE=ANSELE_INIT;
   ANSELF=ANSELF_INIT;
   ANSELG=ANSELG_INIT;

   // initialize ports
   
   PORTA=PORTA_INIT;
   TRISA=TRISA_INIT;

   PORTB=PORTB_INIT;
   TRISB=TRISB_INIT;

   PORTC=PORTC_INIT;
   TRISC=TRISC_INIT;
   
   PORTD=PORTD_INIT;
   TRISD=TRISD_INIT;
   
   PORTE=PORTE_INIT;
   TRISE=TRISE_INIT;

   PORTF=PORTF_INIT;
   TRISF=TRISF_INIT;
   
   PORTG=PORTG_INIT;
   TRISG=TRISG_INIT;

   // initialize PWM steering
   
   APFCON = 0b01000101;
   
   PSTR1CON = 0;
   PSTR2CON = 0;
   PSTR3CON = 0;
   
}

// This function accepts an 8-bit value as an index for a lookup table.
// The table sets a 10-bit value into all the CCPRL registers
void GetCCPVal(unsigned char speed)
{ 
     int iTemp;
     
     // 10 bits of speed data from table  
     iTemp = (CCP_Values[speed]);
     // LSbit in DC2BO
     DC2B0  = iTemp & 0x01;
     
     // next bit in DC2B1
     iTemp = iTemp >> 1;
     DC2B1  = iTemp & 0x01;
     
     // 8 MSbits in CCPRxL
     iTemp = (iTemp >> 1) & 0x00FF;
     CCPR1L = iTemp;
     CCPR2L = iTemp;
     CCPR3L = iTemp;
     
     if(motor_type == BLDC)
     {
	     // transfer DCxB1:0 to other CCPxCON registers
	     iTemp = CCP2CON;
	     CCP1CON = iTemp;
	     CCP3CON = iTemp;
	 }  
}
   
unsigned char FindTableIndex(unsigned int duty_cycle)
{
   // find the closest table index for the supplied value in the table
   // not very efficient but it's only performed during initialization
   unsigned char index;
   
   for(index = 0; CCP_Values[index]<duty_cycle; index++);
   return index;
}
