/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : W.R.Brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : F1_LVDemo_Main.h                           *
*     Date                 : 2011/03/23                                 *
*                                                                       *
*     Other Files Required :                                            *
*                            SelfTest.c                                 *
*                       Common to all motors                            *
*                            F1947_Init_CCP.c                           *
*                            EEData.c                                   *
*                            PC-ADC_SpeedManager_Combo.c                *
*                            Interrupts.c                               *
*                            LinearSpeedProfile_16K.c                   *
*                       Brushed Motor Control                           *
*                            BDC.c                                      *
*                            PAN14_Motor.c                              *
*                       Stepper Motor Control                           *
*                            Stepper.c                                  *
*                            Bipolar_ISR.c                              *
*                            Unipolar_ISR.c                             *
*                            Leadshine_39HS_Motor.c                     *
*                            Leadshine_42HS03_Motor.c                   *
*                            SinusoidalSpeedProfile_Unipolar.c          *
*                            SinusoidalSpeedProfile_Bipolar.c           *
*                       Brushless Motor Control                         *
*                            BLDC_ISR_Combo_CCP.c                       *
*                            BrakeControl.c                             *
*                            DirectBitDrivers_Combo.c                   *
*                            EBM_Motor.c                                *
*                            Stall_CCP.c                                *
*                            Startup_CCP_OpenLoop.c                     *
*                       Monitor Programs                                *
*                            Monitor.c                                  *
*                            UartInterface.c                            *
*                            MonitorTests.c                             *
*                       Driver Programs                                 *
*                            i2c.c                                      *
*                            input.c                                    *
*                            lcd2.c                                     *
*                            mcp9800.c                                  *
*                            rtcc.c                                     *
*                            UNIO_support.c                             *
*                            UNIO PIC16 Protocol.c                      *
*                       Headers                                         *
*                            GenericTypeDefs.h                          *
*                            i2c.h                                      *
*                            input.h                                    *
*                            lcd2.h                                     *
*                            mchp_support.h                             *
*                            mcp9800.h                                  *
*                            rtcc.h                                     *
*                            seven_seg.h                                *
*                            UNIO PIC16.h                               *
*                            UNIO_support.h                             *
*                            Monitor.h                                  *
*                            MonitorTests.h                             *
*                            UartInterface.h                            *
*                            1947_DRIVER.h                              *
*                            BDC.h                                      *
*                            BrakeControl.h                             *
*                            EBM_Motor.h                                *
*                            EEData.h                                   *
*                            Init.h                                     *
*                            Leadshine_39HS_Motor.h                     *
*                            Leadshine_42HS03_Motor.h                   *
*                            Main.h                                     *
*                            PAN14_Motor.c                              *
*                            SpeedManager.h                             *
*                            Startup.h                                  *
*                            Stepper.h                                  *
*                                                                       *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech 9.82                               *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    The Main routine handles Power Up and the Timebase manager.                                     //
//    Other system functions invoked by the Timebase manager are modular such that a particular       //
//    mode of operation can be selected by building the project from the Initialization, Startup,     //
//    Speed Manager and Interrupt Service files with the desired features.                            //
//    For example, to build a controler that uses the PIC16F1937, the CCP to time commutation, and    //
//    the PC for combination open loop and closed loop speed control, then include the Init, Speed    //
//    Manager, Startup, and Interrupt files with those features in the name.                          //
//    For more details on system operation see the header information in the various files            //
//    associated with this project.                                                                   //
//    Main also holds the calling routines to lookup PWM duty cycles from the SpeedProfile array.     //
//                                                                                                    //                                                                                           //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*                
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment

* w.r.brown            2012.04.13  V 2.7 - Corrected comments. Corrected Startup_CCP_OpenLoop.c
* w.r.brown            2012.04.05  V 2.6a- Changed temperature self-test lower limit to 5 deg C.
* w.r.brown            2011.08.23  V 2.6 - Corrected display mode to include Bipolar RPM
* w.r.brown            2011.08.23  V 2.5 - Added restore of non-volatile defaults including Unipolar
*                                          and Bipolar CCP value tables.
* w.r.brown            2011.07.20  V 2.4 - Changed MODE_HOLD to MODE_ADDON
*                                          Changed startup from semiphore driven to state machine
* w.r.brown            2011.07.12  V 2.3 - RPM display mode only for BLDC motor
*                                          Changed BLDC.h to Main.h (support for all motors now)
*                                          Added motor_type to Monitor Test7 list.
* w.r.brown            2011.06.30  V 2.2 - Combined calls to ControlSlowStart, ControlStartup, and StallControl
*                                          in StartupControl().
* w.r.brown            2011.06.21  V 2.1 - Added automatic board detection and self test.
* w.r.brown            2011.03.23  V 2   - Derived from F1947_Combined_Main.c and
*                                          BLDC_Main.c (from F1937/SourceCode) 
*******************************************************************************************************/
#define MAIN_VERSION "2.7";

#include <xc.h>
#include "GenericTypeDefs.h"
#include "Main.h"
#include "EEData.h"
#include "Init.h"
#include "SpeedManager.h"
#include "Startup.h"
#include "BrakeControl.h"

#include "lcd2.h"
#include "rtcc.h"
#include "input.h"
#include "i2c.h"
#include "mcp9800.h"
#include "Monitor.h"
#include "MonitorTests.h"
#include "BDC.h"
#include "UNIO_support.h"
#include "Stepper.h"

/************************************************************************
* variable definitions                                                  *
*************************************************************************/
const char rsMainVersion[] = MAIN_VERSION;

unsigned char comm_state;

doublebyte TMR1_comm_time;
volatile doublebyte InputData;

char slow_start_events;
unsigned int TMR0_run_state_timer;
unsigned char TMR0_stall_timer;
unsigned char TMR0_duty_timer;
unsigned char TMR0_supply_timer;
unsigned char TMR0_stallcheck_timer;
unsigned int  TMR0_button_held_timer;
volatile unsigned char TMR0_temperature_timer;
unsigned char timebase_10ms;
unsigned char timebase_duty_count;

bit TMR0_run_state_flag;
bit TMR0_duty_flag;
bit TMR0_button_flag;
bit TMR0_temperature_flag;
bit TMR0_brake_flag;
bit TMR0_display_flag;
bit startup_complete_flag;
bit stop_flag;
bit run_flag;
bit rising_bemf_flag;
bit measure_bemf_flag;
bit startup_in_progress;
bit SpeedReady;
bit OffsetReady;
bit GetCommTime;
bit BalanceReady;
bit BrakeRequest;
bit ReverseDirection;
bit rpm_thousands_flag;
bit default_reset_flag;

int zc_error;
int temp;
doublebyte expected_zc;
doublebyte zc;
doublebyte comm_after_zc;
unsigned char ramped_speed;

volatile char low_startup_dutycycle;
volatile char startup_dutycycle;
volatile char hi_startup_dutycycle;
volatile unsigned int startup_comm_time;
volatile unsigned int max_timer1_preset;
volatile unsigned int CommTime;

extern enum {high_res_setup,zero_detect,commutate}isr_state;
extern bit OpenLoopFlag;
extern int CommOffset;
extern char cTemp;

extern const unsigned char NumberOfPhases;
unsigned char Poles;
unsigned char blanking_count;
unsigned char SlowStartDwell;
unsigned char SlowStartSteps;
unsigned char OpenLoopDwell;
unsigned char error_scale;
unsigned char speed_request_off;
unsigned char speed_request_on;
unsigned char upward_high_resolution_entry;
unsigned char upward_high_resolution_exit;
unsigned char downward_high_resolution_entry;
unsigned char downward_high_resolution_exit;
unsigned char speed_control_average_factor;
unsigned char BrakeLevel;
unsigned char stepSize;
unsigned char DutyScale;
unsigned char MinDuty;
unsigned char MicroStepSize;
unsigned long SpeedScalingConst;
unsigned long SpeedConstTimer;

volatile char blink;

int         pot_value;
int         rpm;
event_t     btn_event;
char        time_set;
time_t      t;
mode_t      display_mode;
mode_t_incr time_incr_mode;
mode_lcd    lcd_mode;
mode_motor  motor_type;
mode_run    run_mode;
mode_step   step_mode;
BCD_TYPE    bcd_data;
BCD_TYPE_2  bcd_data_2;
BOOL        time_colon;
UINT8       am_pm;
UINT16      battery_voltage;
UINT8       battery_timer;
bit         battery_valid;
bit         LCDNotSuppressed;
unsigned char   add_on_revision;

/************************************************************************
* prototypes                                                            *
*************************************************************************/
void DemoTasks(void);
void display_time(BOOL);
void display_rpm(int r);
void display_temp(BOOL);
void Update_LCD(void);
void acquire_time(BOOL Mode12Hour);
void acquire_temperature(BOOL ModeF);
void acquire_display_data(void);
unsigned long GetRPM(void);
void display_motor_type(void);
void self_test(void);
void DetermineAddOnType(void);
void display_default(void);

/************************************************************************
*                                                                       *
*                              M A I N                                  *
*                                                                       *
*************************************************************************/
void main(void)
{
    Init_IO();
    i2c_init();
    lcd_init();	
    rtcc_init();
    mcp9800_init();
   
    InitSystem();
    DetermineAddOnType();
    step_mode = WAVE;
    lcd_mode = LCD_WAIT;
    battery_timer = 1;
    time_set = 0;
    NV_Variables_Read();
    TMR1_comm_time.word = 1;
    SpeedManagerInit();
       
    while(1) {
        CLRWDT();
        if(stop_flag) InitSystem();
        TimeBaseManager();
        RunControl();
        SpeedManager();
        //BrakeControl();
        DemoTasks();
                	         
        // Run and loop speed indicator
        RunLED = !RunLED;
    }   

} // end main

/************************************************************************
*                         E N D   M A I N                               *
*************************************************************************/

/************************************************************************
*                                                                       *
*      Function:       TimeBaseManager                                  *
*                                                                       *
*      Description:     manage time base timer TIMER0 @ 16.3ms          *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  The TimeBaseManager() is called in the main loop. There are several  *
*  timers in the system which are all controlled by the TimeBaseManager.*
*  The TimeBaseManager() base time resolution is 10mS set by the        * 
*  constant TIMEBASE_LOAD_10ms. All other timers inherit this           *
*  resolution.                                                          *
*  The TimeBaseManager() sets all subroutine timer flags which each     *
*  subroutine then receives as a signal to update their respective      *
*  timers.                                                              *
*                                                                       *
*************************************************************************/
void TimeBaseManager(void) 
{
	if(T0IF) 
   {
		T0IF = 0;
		TMR0 += TIMEBASE_MANAGER_RELOAD_COUNT;
      
      // count down to full 10 ms 
		if(--timebase_10ms) return;
		
		// 10ms TIMEBASE timer
		//--------------------------------------------------------------------------
		timebase_10ms = TIMEBASE_LOAD_10ms;
      
      // run state flag sets the time tick for warmup, slow start, startup, and stall detect
      TMR0_run_state_flag = 1;
   
      // TMR0 duty ramp timer update
      //    "duty" refers to the PWM duty cycle. The PWM duty cycle
      //    is determined by getting a value from the SpeedManager()
      //    The duty cycle change rate is limited by this
      //    timer which permits one increment/decrement of the duty cycle
      //    register every N mS (set by the TIMEBASE_DUTY_RAMP constant).
      TMR0_duty_flag = 1;
      TMR0_brake_flag = 1;
      TMR0_button_flag = 1;
      TMR0_temperature_flag = 1;
      TMR0_display_flag = 1;
   }
}

/************************************************************************
*                         Demo Routines                                 *
*************************************************************************/

void DemoTasks(void)
{
	btn_event = input_event();
//	if(display_mode != MODE_ADDON)  // include this condition for minimum current in ADDON display mode
		pot_value = input_pot();

 	switch(btn_event)
 	{
     	case BUTTON_UP: break;
     	case BUTTON_DOWN: // button is sensed as held down
     	    if(display_mode == MODE_TIME && time_set == 0)
     	    {
        	    if(TMR0_button_flag)
	        	    if(TMR0_button_held_timer-- == 0)
	        	    {
		        	    // enter time set mode when the button is held long enough
	            	    time_set = 1;
	            	    // time set routine will decrement TMR0_button_held_timer to zero
	            	    // and make the first time adjustment immediately
	            	    TMR0_button_held_timer++;
	            	    // read the current time into t and strip off seconds
						time(&t);
						t = t - (t % 60);
						time_incr_mode = INCR_MINUTES;
	            	}
	            else
	            	TMR0_button_flag = 0;
            }
            if(display_mode == MODE_ADDON)
            {
                if (TMR0_button_flag)
                {
                    if(TMR0_button_held_timer-- == 0)
                    {
                        // reset variables to factory defaults for attached motor
                        set_defaults();
                        LCDNotSuppressed = 1;
                        default_reset_flag = 1;
                    }
                }
                TMR0_button_flag = 0;
            }
     	    break;
     	case BUTTON_PRESSED: // button just became pressed
        	if(display_mode == MODE_TIME)
        	{	// set the held-button timer for 0.35 sec (0.4/Timebase manager resolution)
	        	// button must be held down for this amount of time to enter time set mode
        	    TMR0_button_held_timer = 35;
        	}    
        	if(display_mode == MODE_ADDON)
        	{	// set the held-button timer for 5 sec (5.0/Timebase manager resolution)
	        	// button must be held down for this amount of time to force defaults reload
        	    TMR0_button_held_timer = 500;
        	}    
     	    break;
     	case BUTTON_RELEASED: // on release, next display mode (unless we were setting the time)
     	    if(time_set == 0)
     	    {
         	    switch(display_mode)
         	    {
             	    case MODE_TIME:
                        display_mode = MODE_TIME_24;
             	        break;
             	    case MODE_TIME_24:
             	        // temperature display changes occur every 0.5 sec
             	        // timer is set to 1 so the initial display occurs immediately
             	        TMR0_temperature_timer = 1;
                        display_mode = MODE_TEMPERATURE;
             	        break;
             	    case MODE_TEMPERATURE:
             	        // timer is set to 1 so the initial display occurs immediately
             	        TMR0_temperature_timer = 1;
                        display_mode = MODE_TEMPERATURE_F;
             	        break;
             	    case MODE_TEMPERATURE_F:
                        display_mode = MODE_POT;
             	        break;
             	    case MODE_POT:
             	        display_mode = MODE_RPM;
             	        if((motor_type != BLDC) && (motor_type != UNIPOLAR) && (motor_type != BIPOLAR))
             	        {
             	            display_mode = MODE_ADDON; 
             	    	    LCDNotSuppressed = 1;
             	    	}
             	        break;
             	    case MODE_RPM:
             	        display_mode = MODE_ADDON;
             	        // LCDNotSuppressed flag causes display to update the first time only
             	        // and then further display updates are suppressed until the mode changes.
             	    	LCDNotSuppressed = 1;
             	        break;
             	    case MODE_ADDON:
             	    	/* this section was used to step through and test display of all the different types
             	    	LCDNotSuppressed = 1;
             	    	switch (motor_type)
             	    	{
	             	    	case NONE:
	             	    		motor_type = BLDC;
	             	    		break;
	             	    	case BLDC:
	             	    		motor_type = UNIPOLAR;
	             	    		break;
	             	    	case UNIPOLAR:
	             	    		motor_type = BIPOLAR;
	             	    		break;
	             	    	case BIPOLAR:
	             	    		motor_type = BDC;
	             	    		break;
	             	    	default:
	             	    	case BDC:
	             	    		motor_type = NONE;
             	        		display_mode = MODE_TIME;
	             	    		break;
	             	    }
	             	    */
	             	    display_mode = MODE_TIME;
	             	    break;
             	    default:
             	        display_mode = MODE_TIME;
             	        break;
             	}
         	}
         	else
         	    // exit time set mode on button release
         	    // time resumes from where it was left
         		time_set = 0;
     	    break;
     	default: break;
   }
        
   // Time set increments the clock time by one minute for every held button interval
   // when the minutes reach an even 10 then the increment is 10 minutes
   // when the minutes reach an even hour then the increment is one hour
   if(time_set) // if we are setting the time ...
   {
      // TimeBaseManager sets the TMR0_button_flag every 10 ms.
      if(TMR0_button_flag)
      {
         TMR0_button_flag = 0;
         if(TMR0_button_held_timer-- == 0)
         {
            // increments occur every 0.35 second while the button is held
            TMR0_button_held_timer =  35;
            // adjust the time
            switch (time_incr_mode)
            {
               case INCR_MINUTES:
                  t += 60;             // increment minutes while not even 10
                  if ((t % 600)==0)    // check even 10
                  	time_incr_mode = INCR_TENS;
                  break;
               case INCR_TENS:
                  t += 600;             // increment by 10 minutes while not even hour
                  if ((t % 3600)==0)    // check even 60
                  	time_incr_mode = INCR_HOURS;
                  break;
               case INCR_HOURS:
               default:
                  t += 3600;  // increment hour
                  break;
            }
		  		// show the current setting
            rtcc_set(&t);
         }
      }
	}
	else // we are not setting the time
   {
      // update the time
      rtcc_handler();
   }

   // acquire and display data every 10 ms
   switch(lcd_mode)
   {
      default:
      case LCD_ACQUIRE:
          if(TMR0_display_flag)
          {
              TMR0_display_flag = 0;
              acquire_display_data();
      	      lcd_mode = LCD_ARM;
      	  }
      	  break;	
      case LCD_ARM:
           if(!WA) lcd_mode = LCD_WAIT;
      	   break;
      case LCD_WAIT:
          // catch WA low to high transition then display acquired data
          if(WA)
          {  
              Update_LCD();
              lcd_mode = LCD_ACQUIRE;
          }
          break;
   }
}
/*************************************/
/* LCD HELPERS                       */
/*************************************/
void display_temp(BOOL ModeF)
{
    // display temperature in degrees C or F
    // temperature is assumed to have been acquired, converted, and stored in bcd_data 
    UINT8 Indicator;
    
    lcd_display_LS_digits(bcd_data,2);
    if(ModeF)
        Indicator = LETTER_F;
    else
        Indicator = LETTER_C;
    lcd_display_character(6, Indicator);    
    lcd_display_character(5, SYMBOL_deg);    
    DP3 = 0;
    DP2 = 0;
    DP1 = 1;
    P4 = 0;
    F_1 = 0;
    F_2 = 1;
    F_3 = 0;
    F_4 = 0;
    LOGO = 0;
}

void display_rpm(int r)
{
    static int counter = 0;
    static int integrator=0;
    int average;
    static int v = 0;
    
    // scale down
    if((DP2 == 1) && (!rpm_thousands_flag))
    	integrator *= 10;
    // scale up
    if((DP2 == 0) && (rpm_thousands_flag))
    	integrator /= 10;
    	
    // low pass filter
    integrator += r;
    average = integrator / 8;
    integrator -= average;
    
    // decimate
    if(counter-- == 0)
    {
        counter = 10;
        v = average;
    }

    // display
    display_int(v);
    DP1 = 0;
    DP3 = 0;
    if(rpm_thousands_flag)
    {
	    DP2 = 1;
	    lcd_display_character(5, LETTER_t);
	    lcd_display_character(6, LETTER_h);
	}
	else
	{
	    DP2 = 0;
	    lcd_display_character(5, BLANK);
	    lcd_display_character(6, BLANK);
    }
    P4 = 0;
    F_1 = 0;
    F_2 = 0;
    F_3 = 0;
    F_4 = 1;
    LOGO = 0;
}
    
void display_pot(int p)
{
    static int counter = 0;
    
    //decimate
    if(counter-- == 0)
    {
        counter = 10;    
        display_int(p);
        lcd_display_character(5, LETTER_P);
        lcd_display_character(6, LETTER_t);
        DP1 = 0;
        DP2 = 0;
        DP3 = 0;
        P4 = 0;
        F_1 = 0;
        F_2 = 0;
        F_3 = 3;
        F_4 = 0;
        LOGO = 0;
     }
}

void display_int(int t)
{
    BCD_TYPE bcd;
    
    bcd.digit0 = t %10;
    t /= 10;
    bcd.digit1 = t % 10;
    t /= 10;
    bcd.digit2 = t % 10;
    t /= 10;
    bcd.digit3 = t%10;
    
    lcd_display_LS_digits(bcd,1);   
}    

void display_time(BOOL Mode12Hour)
{
    // display time in 12 hour AM/PM or 24 hour format
    // bcd_data, am_pm, and time_colon are assumed to have been set by previous call to acquire_time()
        
    if (Mode12Hour)
    {
        lcd_display_LS_digits(bcd_data,3);
        lcd_display_character(5,am_pm);
        lcd_display_character(6,BLANK);
        if (time_colon)
            P4 = 1;
        else
            P4 = 0;
    }
    else
    {
        lcd_display_digits(bcd_data);
        lcd_display_digits_2(bcd_data_2);
        P4 = 1;
    }    
        
    DP1 = 0;
    DP2 = 0;
    DP3 = 0;
    F_1 = 1;
    F_2 = 0;
    F_3 = 0;
    F_4 = 0;
    LOGO = 0;
}

void display_battery(void)
{
    // indicate battery status from previously measured battery voltage
    battery_valid = 0;
    
    // segments come on at the thresholds but don't go off until below the
    // threshold minus some hysteresis
    if(battery_voltage > BATTERY_EMPTY)
        T1 = 1;
    else
    {
        if (battery_voltage < (BATTERY_EMPTY - BATTERY_HYSTERESIS))
            T1 = 0;
        T2 = 0;
        T3 = 0;
        T4 = 0;
        return;
    }

    if(battery_voltage > BATTERY_1_THIRD)
        T2 = 1;
    else
    {
        if (battery_voltage < (BATTERY_1_THIRD - BATTERY_HYSTERESIS))
            T2 = 0;
        T3 = 0;
        T4 = 0;
        return;
    }

    if(battery_voltage > BATTERY_2_THIRDS)
        T3 = 1;
    else
    {
        if (battery_voltage < (BATTERY_2_THIRDS - BATTERY_HYSTERESIS))
            T3 = 0;
        T4 = 0;
        return;
    }

    if(battery_voltage > BATTERY_FULL)
        T4 = 1;
    else
        if(battery_voltage < (BATTERY_FULL - BATTERY_HYSTERESIS))
            T4 = 0;
            
    return;
}    
    
void Update_LCD(void)
{
    // call this routine upon sensing WA rising edge
    switch(display_mode)
    {
    case MODE_TIME: 
        display_time(TRUE);
        break;
    case MODE_TIME_24: 
        display_time(FALSE);
        break;
    case MODE_TEMPERATURE:
        display_temp(FALSE);
        break;
    case MODE_TEMPERATURE_F:
        display_temp(TRUE);
        break;
    case MODE_RPM:
        display_rpm(rpm);
        break;
    case MODE_POT:
        display_pot(pot_value);
        break;
    case MODE_ADDON:
        //suppress display changes
        if(LCDNotSuppressed)
        {
            if(default_reset_flag)
            {
                display_default();
                default_reset_flag = 0;
            }
            else
                display_motor_type();
            // Clear F_4 and light the LOGO the first time through MODE_ADDON
            LCDNotSuppressed = 0;
            F_4 = 0;
            LOGO = 1;
        }
    default:
        break;
    }
    if(battery_valid) display_battery();

}

void acquire_time(BOOL Mode12Hour)
{
    // acquire the present time and convert to BCD digits. Store result in global bcd_data
    // if Mode12Hour is true then adjust digits and set AM/PM indicator accordingly

    time_t tym;
    struct tm *d;
    
    time(&tym);
    d = gmtime(&tym);
    
    // time_colon is blinking seconds indicator
    time_colon = ((tym % 2) == 1);

    bcd_data.digit0 = d->tm_min % 10;
    bcd_data.digit1 = d->tm_min / 10;
    
    if(Mode12Hour)
    {
        if(d->tm_hour > 11)
        {
            d->tm_hour -= 12;
            am_pm = LETTER_P;  // PM indicator
        }    
        else
        {
            am_pm = LETTER_A;  // AM indicator
        }
    
        if(d->tm_hour == 0) d->tm_hour = 12;
    }
    else
    {
        bcd_data_2.digit0 = d->tm_sec % 10;
        bcd_data_2.digit1 = d->tm_sec / 10;
    }
    
    bcd_data.digit2 = d->tm_hour % 10;
    bcd_data.digit3 = d->tm_hour/10;
}

void acquire_temperature(BOOL ModeF)
{
    // acquire temperature from remote device and convert and store in global bcd_data
    int tmp;
    
    tmp = mcp9800_get_temp();
    // assumes t is the temperature in degrees C * 10
    if(ModeF)
        tmp = ((9*tmp)/5)+320;

    bcd_data.digit0 = tmp%10;
    tmp /= 10;
    bcd_data.digit1 = tmp%10;
    tmp /= 10;
    bcd_data.digit2 = tmp%10;
    tmp /= 10;
    bcd_data.digit3 = tmp%10;    
}

void acquire_display_data(void)
{
    // acquire and convert selected display data
    // conversions are made at loop rate so data is
    // ready to be written in the short LCD WA window.
    
    switch(display_mode)
    {
        case MODE_TIME:
            acquire_time(TRUE);
            break;
        case MODE_TIME_24: 
            acquire_time(FALSE);
            break;
        case MODE_TEMPERATURE:
        case MODE_TEMPERATURE_F:
            // flag is set every 10 ms by TimeBaseManager
            if(TMR0_temperature_flag)
            {
                TMR0_temperature_flag = 0;
                // acquire new temperature every .5 sec
                if(--TMR0_temperature_timer == 0)
                {
               		acquire_temperature(display_mode == MODE_TEMPERATURE_F);
          			TMR0_temperature_timer = 50;
               	}
           	}
            break;
        case MODE_RPM:
            rpm = GetRPM();
            // set or clear scaling flag with 500 rpm hysteresis
            if(rpm > 3000)
            	rpm_thousands_flag = 1;
            else
            	if(rpm < 2500) rpm_thousands_flag = 0;
            if(rpm_thousands_flag)rpm /= 10;
            break;
        default:
            break;
    }
    // this routine is visited at the LCD refresh rate which is about 3.9 ms
    // the battery voltage is read and displayed about every 1 second
    // which occurs every 256 LCD frames.
    if(--battery_timer == 0)
    {
	    if(display_mode != MODE_ADDON)
        	battery_voltage = get_adc(BATTERY_CHANNEL);
        // set flag to cause battery display update
        battery_valid = 1;
   	}
}

/************************************************************************
*                                                                       *
*      Function:       GetRPM                                           *
*                                                                       *
*      Description:     calculate the motor RPM from the                *
*                       commutation time                                *
*                                                                       *
*      Parameters:                                                      *
*      Return value:    RPM                                             *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

unsigned long GetRPM(void)
{
    unsigned long ul_rpm;
    
    if(!run_flag) // the motor is not running, return 0
        return 0;
    
    if(motor_type == BLDC)
    {    
        ul_rpm = 60L * 1000L * 1000L;
        ul_rpm /= Poles * NumberOfPhases * (unsigned int)TMR1_comm_time.word;
    }
    else
    {  // stepper
        ul_rpm =  SpeedConstTimer>>stepSize ;
        ul_rpm /= TMR1_comm_time.word;
    }
    return ul_rpm;
}    

void display_motor_type(void)
{
	switch (motor_type)
	{
		case NONE:
		    lcd_display_character(1,DIGIT_0);
            lcd_display_character(2,LETTER_P);
            lcd_display_character(3,LETTER_E);
            lcd_display_character(4,LETTER_n);
            break;
        case BLDC:           
		    lcd_display_character(1,LETTER_b);
            lcd_display_character(2,LETTER_L);
            lcd_display_character(3,LETTER_d);
            lcd_display_character(4,LETTER_c);
            break;
        case UNIPOLAR:           
		    lcd_display_character(1,LETTER_U);
            lcd_display_character(2,LETTER_S);
            lcd_display_character(3,LETTER_t);
            lcd_display_character(4,LETTER_P);
            break;
        case BIPOLAR:           
		    lcd_display_character(1,LETTER_b);
            lcd_display_character(2,LETTER_S);
            lcd_display_character(3,LETTER_t);
            lcd_display_character(4,LETTER_P);
            break;
	    case BDC:           
            lcd_display_character(1,BLANK);
		    lcd_display_character(2,LETTER_b);
            lcd_display_character(3,LETTER_d);
            lcd_display_character(4,LETTER_c);
            break;
        case TEST:
		    lcd_display_character(1,LETTER_t);
            lcd_display_character(2,LETTER_E);
            lcd_display_character(3,LETTER_S);
            lcd_display_character(4,LETTER_t);
            break;
        default:
            lcd_display_character(1,LETTER_F);
		    lcd_display_character(2,LETTER_A);
            lcd_display_character(3,LETTER_L);
            lcd_display_character(4,LETTER_t);
            break;
    }
    
    if(add_on_revision < 99)
    {
    	bcd_data_2.digit0 = add_on_revision % 10;
    	bcd_data_2.digit1 = add_on_revision / 10;
    	lcd_display_digits_2(bcd_data_2);
	}
}

void DetermineAddOnType(void)
{
	INT16 adc_value;
	
    // assume that an add-on board is attached
    // these two parameters will be changed if one is not
    LCDNotSuppressed = 1;
    display_mode = MODE_ADDON;
    
	ReadAddOnType();
    switch(motor_type)
    {
        case TEST:
            self_test();
            break;
        case NONE:
        case UNDEFINED:
    		// discharge the input then take the measurement
    		ADCON0bits.CHS = BEMF_REFERENCE;
    		LATF6 = 0;
    		TRISF6 = 0;
    		TRISF6 = 1;
    		adc_value = get_adc(BEMF_REFERENCE);
    		if((adc_value> 0x0070) && (adc_value < 0x0200))
    		{
    			add_on_revision = 1;
    			motor_type = BLDC;
    		}
    		else
    		{
                LCDNotSuppressed = 0;
                display_mode = MODE_TIME;
            }   
    		break;
        default:
            break;
    }
}	

void display_default(void)
{
    lcd_display_character(1,LETTER_d);
    lcd_display_character(2,LETTER_F);
    lcd_display_character(3,LETTER_L);
    lcd_display_character(4,LETTER_t);
}