/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Motor Control             *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : Interrupts.c                               *
*     Date                 : 2011/08/09                                 *
*                                                                       *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech 9.82                               *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Plain vanilla interrupts with two states: Zero_detect and Commutate.                             //
//   This version also includes dynamic blanking.                                                     //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown           2011.07.21  V 2.0 - Complete overhaul
* i. boholteanu       2011.03.30  V 1.0 - first revision
*******************************************************************************************************/
#include <xc.h>
#include "Main.h"

extern doublebyte TMR1_comm_time;
extern bit GetCommTime;
extern volatile unsigned int CommTime;

void brushless_interrupt_handler(void);
void unipolar_interrupt_handler(void);
void bipolar_interrupt_handler(void);

extern mode_motor motor_type;

/************************************************************************
*                                                                       *
*                          I N T E R R U P T                            *
*                                                                       *
*************************************************************************/
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    Motor drive is controlled by interrupts. The number and types of interrupt depend on the        //
//    motor type. See each ISR file for more information.                                             //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
static void interrupt interrupt_handler(void) 
{ 
    switch (motor_type)
    {
        case BLDC:
            brushless_interrupt_handler();
            break;
        case UNIPOLAR:
            unipolar_interrupt_handler();
            break;
        case BIPOLAR:
            bipolar_interrupt_handler();
            break;
        default:
            break;
    }
     // fill request for commutation period
     if(GetCommTime)
     {                   
        CommTime = (unsigned int)TMR1_comm_time.word;
        GetCommTime = 0;
     }
} 
/* end ISR */