/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : SinusoidalSpeedProfile_16K.c               *
*     Date                 : 2011/08/12                                 *
*     SharePoint Version   : 1.1                                        *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Speed lookup tables for 16 kHz PWM frequency. Sinusoidal voltage output.                         //
//   This table is based on Fosc=32 MHz and 1:4 Timer2 prescale resulting in PR2 = 0x7C.              //
//   This table was built by scaling a quarter wave sinusoid to the maximum PWM period.               //
//   The PWM period is set by PR2. However, the duty cycle is potentially a 10 bit value (9 bits      //
//   in this particular case) so the scaling factor is four times the PR2 value.                      //
//                                                                                                    //
//   To keep the values less than 8 bits a scaling factor of 2 was used.                              //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
*******************************************************************************************************/



#include "Main.h"
//***********************************************************************
// Quarter wave sinusoid table 
// 

const unsigned char default_CCP_Val[] = {
0,	    2,	    3,	    5,	    6,	    8,	    9,  	11,	    12,	    14,	    15,	    17,	    18,	    20,	    21,	    23,
25,	    26,	    28,	    29,	    31,	    32,	    34,	    35,	    37,	    38,	    40,	    41,	    43,	    44,	    46,	    47,
49,	    50,	    52,	    53,	    55,	    56, 	58,	    59,	    61,	    62,	    64,	    65,	    67,	    68,	    70,	    71,
73,	    74,	    76,	    77,	    78,	    80, 	81,	    83,	    84,	    86,	    87,	    89,	    90,	    91,	    93,	    94,
96,	    97,	    98,	    100,	101,	103,	104,	106,	107,	108,	110,	111,	112,	114,	115,	116,
118,    119,	121,	122,	123,	125,	126,	127,	129,	130,	131,	132,	134,	135,	136,	138,
139,	140,	141,	143,	144,	145,	146,	148,	149,	150,	151,	153,	154,	155,	156,	157,
159,	160,	161,	162,	163,	164,	166,	167,	168,	169,	170,	171,	172,	173,	175,	176,
177,	178,	179,	180,	181,	182,	183,	184,	185,	186,	187,	188,	189,	190,	191,	192,
193,	194,	195,	196,	197,	198,	199,	200,	201,	202,	203,	204,	204,	205,	206,	207,
208,	209,	210,	210,	211,	212,	213,	214,	214,	215,	216,	217,	218,	218,	219,	220,
220,	221,	222,	223,	223,	224,	225,	225,	226,	227,	227,	228,	229,	229,	230,	230,
231,	232,	232,	233,	233,	234,	234,	235,	235,	236,	236,	237,	237,	238,	238,	239,
239,	240,	240,	241,	241,	241,	242,	242,	243,	243,	243,	244,	244,	244,	245,	245,
245,	245,	246,	246,	246,	247,	247,	247,	247,	248,	248,	248,	248,	248,	248,	249,
249,	249,	249,	249,	249,	249,	250,	250,	250,	250,	250,	250,	250,	250,	250,	250
};

unsigned char CCP_Val[sizeof default_CCP_Val];
