/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : SinusoidalSpeedProfile_Unipolar.c          *
*     Date                 : 2011/08/12                                 *
*     SharePoint Version   : 1.1                                        *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Speed lookup tables for 16 kHz PWM frequency. Sinusoidal voltage output.                         //
//   This table is based on Fosc=32 MHz and 1:4 Timer2 prescale resulting in PR2 = 0x7C.              //
//   This table was built by scaling a quarter wave sinusoid to the maximum PWM period.               //
//   The PWM period is set by PR2. However, the duty cycle is potentially a 10 bit value (9 bits      //
//   in this particular case) so the scaling factor is four times the PR2 value.                      //
//                                                                                                    //
//   To keep the values less than 8 bits a scaling factor of 2 was used.                              //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
*******************************************************************************************************/



#include "Main.h"
//***********************************************************************
// Quarter wave sinusoid table 
// 

const unsigned char Unipolar_CCP_Val[] = {
0,	1,	2,	2,	3,	4,	5,	5,	6,	7,	8,	8,	9,	10,	11,	11,
12,	13,	14,	15,	15,	16,	17,	18,	18,	19,	20,	21,	21,	22,	23,	24,
24,	25,	26,	27,	27,	28,	29,	30,	30,	31,	32,	33,	33,	34,	35,	36,
36,	37,	38,	38,	39,	40,	41,	41,	42,	43,	44,	44,	45,	46,	46,	47,
48,	49,	49,	50,	51,	51,	52,	53,	53,	54,	55,	56,	56,	57,	58,	58,
59,	60,	60,	61,	62,	62,	63,	64,	64,	65,	66,	66,	67,	68,	68,	69,
69,	70,	71,	71,	72,	73,	73,	74,	74,	75,	76,	76,	77,	78,	78,	79,
79,	80,	80,	81,	82,	82,	83,	83,	84,	85,	85,	86,	86,	87,	87,	88,
88,	89,	89,	90,	91,	91,	92,	92,	93,	93,	94,	94,	95,	95,	96,	96,
97,	97,	98,	98,	99,	99,	99,	100,100,101,101,102,102,103,103,104,
104,104,105,105,106,106,106,107,107,108,108,108,109,109,110,110,
110,111,111,111,112,112,112,113,113,113,114,114,114,115,115,115,
115,116,116,116,117,117,117,117,118,118,118,118,119,119,119,119,
120,120,120,120,120,121,121,121,121,121,122,122,122,122,122,122,
123,123,123,123,123,123,123,124,124,124,124,124,124,124,124,124,
124,124,125,125,125,125,125,125,125,125,125,125,125,125,125,125
};
	
