// File: Leadshine_43HS03_Motor.h
//
// Motor parameters for the Leadshine 42HS03 motor supplied with Unipolar Add-on demo board.
//
#ifndef _LEADSHINE_42HS03_MOTOR
#define _LEADSHINE_42HS03_MOTOR

/************************************************************************
* Motor Definitions                                                               
*************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Low speed On-Off limits ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////
//// On-Off limits


#define REQUEST_OFF            5
#define REQUEST_ON             8 
//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////// Speed control averaging factors ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// ADC averaging factor
// Number of samples in the ADC average = 2^ADC_AVG_FACTOR
#define ADC_AVG_FACTOR          2

/////////////////////////////////////////////////////////////////////////////////////////
//		MOTOR PARAMETERS
////////////////////////////////////////////////////////////////////////////////////////

//	Step angle [angle*10]	
#define MOTOR_STEP_ANGLE		18L

// no. of steps in one revolution (one mechanical cycle)
#define STEPS_PER_REVOLUTION      (3600 /MOTOR_STEP_ANGLE)

#define SPEED_CONST_TIMER	(((unsigned long)SYSTEM_FREQUENCY/(STEPS_PER_REVOLUTION * TMR1_PRESCALE))*60)

#define MOTOR_RATED_VOLTAGE		4.6		// [Volts*100]
#define MOTOR_RATED_CURRENT		((MOTOR_RATED_VOLTAGE*10)/MOTOR_RESISTANCE)		// *10 [Amps]
#define MOTOR_RESISTANCE  		460		// [Ohms*100] 
#define POWER_SUPPLY_VOLTAGE	9		// [Volts]


#define LOOK_UP_TABLE_SIZE	256
#define MAX_SPEED_VALUE     255

// without diodes
//#define MAX_RPM                 312L

// with diodes and no series resistance
#define MAX_RPM                 76L

#define MAX_STEPS_PER_MINUTE    (STEPS_PER_REVOLUTION*MAX_RPM)
#define MAX_STEPS_PER_SECOND    (MAX_STEPS_PER_MINUTE/60L)
#define MIN_TIMER1_COUNT        (TMR1_FREQUENCY/MAX_STEPS_PER_SECOND)

#define SPEED_SCALING_CONST     MIN_TIMER1_COUNT*MAX_SPEED_VALUE

// PWM duty cycle is a linear function of the speed control input
// MIN_DUTY is the smallest duty cycle that will move the motor
// MAX_DUTY is the largest duty cycle permitted at the maximum motor speed
//   y is an 8-bit number applied to a 9-bit field: CCPRxL[6:0] = y[7:1], DBx1 = y[0]
// The transfer function of speed input to desired duty cycle output follows the linear relationship
//   y = (256*mX/256) + B)
//   Where:
//     X = speed input
//     m = DUTY_SCALE
//     B = MIN_DUTY
//     y = output to CCPRxL to set the duty cycle
//     m is scaled up by 256 to allow fractional values of m.

// without diodes
//#define MIN_DUTY                144L
//#define MAX_DUTY                ((PWM_PERIOD+1)*2)

// with diodes and no series resistance
#define MIN_DUTY                56L
#define MAX_DUTY                (PWM_PERIOD+1)

#define DUTY_SCALE              (MAX_DUTY-MIN_DUTY+1)

// default power of 2 micro steps in each full step when microstepping is selected
// i.e. number of micro steps in each step angle = 2^MICRO_STEP_SIZE
#define MICRO_STEP_SIZE         2

#endif